function plotmop(testname, mop)
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% --------
% License:
% --------
% This program is to be used under the terms of the GNU General Public License 
% (http://www.gnu.org/copyleft/gpl.html).
% Author: Asad Mohammadi
% Email : asadmoh1985@gmail.com
% Copyright notice: (c) 2014 Asad Mohammadi
%In this function, plots of the different test problems are
%drawn

switch lower(testname)
    case 'kno1'
        plotkno1(mop);
    case 'zdt1'
        plotzdt1(mop);
    case 'zdt2'
        plotzdt2(mop);
    case 'zdt3'
        plotzdt3(mop);
    case 'zdt4'
        plotzdt4(mop);
    case 'zdt5'
        plotzdt5(mop);        
    case 'zdt6'
        plotzdt6(mop);
     case 'dtlz1_2d'
        plotdtlz1_2d(mop);
     case 'dtlz1_3d'
        plotdtlz1_3d(mop);
    case 'dtlz2'
        plotdtlz2(mop);
    otherwise 
        error('Undefined test problem name');                
end 
end

function plotzdt1(mop)
    x1 = [0:0.01:1]';
    points = [x1 zeros(length(x1), mop.pd - 1)];
    h = zeros(length(x1), 1);
    for i=1:size(points,1)
        y = mop.func(points(i, :));
        h(i) = y(2);
    end
    plot(x1, h,'LineWidth',2.3);    
end


function plotzdt2(mop)
    x1 = [0:0.01:1]';
    points = [x1 zeros(length(x1), mop.pd - 1)];
    h = zeros(length(x1), 1);
    for i=1:size(points,1)
        
        y = mop.func(points(i, :));
        h(i) = y(2);
    end
    plot(x1, h,'LineWidth',2.3);
end


function plotzdt3(mop)
    x1 = [0:0.001:1]';
    points = [x1 zeros(length(x1), mop.pd - 1)];
    h = zeros(length(x1), 1);
    for i=1:size(points,1)
        y = mop.func(points(i, :));
        h(i) = y(2);
    end
    
    newPoints = [];
    pairs = [x1 h];
    last = pairs(1, :);
    for i=2:size(pairs,1)
        diff = pairs(i, :) - last;
        if (diff(1) * diff(2) <= 0)
            last = pairs(i, :);
            newPoints = [newPoints ; pairs(i, :)];
        else
            if(length(newPoints) ~= 0)
                hold on;
                plot(newPoints(:,1), newPoints(:, 2),'LineWidth',2.3);
                newPoints = [];
            end
        end
    end
    
   
end


function plotzdt4(mop)
    x1 = [0:0.01:1]';
    points = [x1 zeros(length(x1), mop.pd - 1)];
    h = zeros(length(x1), 1);
    for i=1:size(points,1)
        y = mop.func(points(i, :));
        h(i) = y(2);
    end
    plot(x1, h,'LineWidth',2.3);
end

function plotzdt5(mop)
    f1 = [1:1:31]';
    
    h = zeros(length(f1), 1);
    for i=1:length(f1)
        
        h(i) = 10/f1(i);
    end
    scatter(f1(:), h(:));
    hold on;
    plot(f1, h,'LineWidth',2.3);
end



function plotzdt6(mop)
    x1 = [0:0.0001:1]';
    points = [x1 zeros(length(x1), mop.pd - 1)];
    h = zeros(length(x1), 2);
    for i=1:size(points,1)
        h(i,:) = mop.func(points(i, :));
    end
    plot(h(:,1), h(:,2),'LineWidth',2.3);
end

function plotdtlz1_2d(mop)
    x = [0:0.1:1];
    y = 1 - x;    
    y(find(y<=0)) = NaN;
    plot(x,y,'LineWidth',2.3);
end

function plotdtlz1_3d(mop)
    [x y] = meshgrid(0:0.1:1);
    z = 1 - x - y;    
    z(find(z<=0)) = NaN;
    mesh(x,y,z,'LineWidth',2.3);
    
   
end

function plotdtlz2(mop)
    [x y] = meshgrid(0:0.05:1);
    inds = find(1 - x.^2 - y.^2 < 0);
    x(inds) = NaN;
    y(inds) = NaN;
    z = sqrt(1 - x.^2 - y.^2);
    mesh(x,y,z,'LineWidth',2.3);
end



